# Generated by Django 2.2.6 on 2020-01-28 18:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0002_translatablechar'),
    ]

    operations = [
        migrations.CreateModel(
            name='RequestTrackExecInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255)),
                ('start_time', models.DateTimeField(blank=True, null=True)),
                ('end_time', models.DateTimeField(blank=True, null=True)),
                ('milliseconds', models.FloatField(blank=True, default=None, null=True)),
                ('request_track', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='exec_infos', to='avishan.RequestTrack')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
