# Generated by Django 3.0.3 on 2020-02-27 20:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('avishan', '0006_auto_20200219_0421'),
    ]

    operations = [
        migrations.CreateModel(
            name='VisitorKey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('last_used', models.DateTimeField(blank=True, default=None, null=True)),
                ('last_login', models.DateTimeField(blank=True, default=None, null=True)),
                ('last_logout', models.DateTimeField(blank=True, default=None, null=True)),
                ('key', models.CharField(max_length=255, unique=True)),
                ('user_user_group', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='avishan.UserUserGroup')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
